@@ -1,6 +1,10 @@
+# frozen_string_literal: true
+
 module Clamp
   module Attribute
 
+    # Methods to generate attribute accessors.
+    #
     module Declaration
 
       protected
@@ -25,6 +29,8 @@ module Clamp
       end
 
       def define_default_for(attribute)
+        return false if attribute.default_value.nil?
+
         define_method(attribute.default_method) do
           attribute.default_value
         end
