@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 
 require "spec_helper"
 
@@ -30,18 +31,35 @@ describe Clamp::Command do
     end
 
     it "executes the #execute method" do
-      expect(stdout).to_not be_empty
+      expect(stdout).not_to be_empty
     end
 
   end
 
   describe ".option" do
 
-    it "declares option argument accessors" do
-      command.class.option "--flavour", "FLAVOUR", "Flavour of the month"
-      expect(command.flavour).to eql nil
-      command.flavour = "chocolate"
-      expect(command.flavour).to eql "chocolate"
+    context "when regular" do
+
+      before do
+        command.class.option "--flavour", "FLAVOUR", "Flavour of the month"
+      end
+
+      context "when not set" do
+        it "equals nil" do
+          expect(command.flavour).to be_nil
+        end
+      end
+
+      context "when set" do
+        before do
+          command.flavour = "chocolate"
+        end
+
+        it "equals the value" do
+          expect(command.flavour).to eq "chocolate"
+        end
+      end
+
     end
 
     context "with type :flag" do
@@ -50,9 +68,20 @@ describe Clamp::Command do
         command.class.option "--verbose", :flag, "Be heartier"
       end
 
-      it "declares a predicate-style reader" do
-        expect(command).to respond_to(:verbose?)
-        expect(command).to_not respond_to(:verbose)
+      describe "predicate-style reader" do
+
+        it "exists" do
+          expect(command).to respond_to(:verbose?)
+        end
+
+      end
+
+      describe "regular reader" do
+
+        it "does not exist" do
+          expect(command).not_to respond_to(:verbose)
+        end
+
       end
 
     end
@@ -60,17 +89,24 @@ describe Clamp::Command do
     context "with explicit :attribute_name" do
 
       before do
-        command.class.option "--foo", "FOO", "A foo", :attribute_name => :bar
+        command.class.option "--foo", "FOO", "A foo", attribute_name: :bar
       end
 
       it "uses the specified attribute_name name to name accessors" do
         command.bar = "chocolate"
-        expect(command.bar).to eql "chocolate"
+        expect(command.bar).to eq "chocolate"
       end
 
-      it "does not attempt to create the default accessors" do
-        expect(command).to_not respond_to(:foo)
-        expect(command).to_not respond_to(:foo=)
+      describe "default reader" do
+        it "does not exist" do
+          expect(command).not_to respond_to(:foo)
+        end
+      end
+
+      describe "default writer" do
+        it "does not exist" do
+          expect(command).not_to respond_to(:foo=)
+        end
       end
 
     end
@@ -87,7 +123,7 @@ describe Clamp::Command do
       end
 
       it "sets the specified default value" do
-        expect(command.port).to eql 4321
+        expect(command.port).to eq 4321
       end
 
     end
@@ -95,11 +131,11 @@ describe Clamp::Command do
     context "with :default value" do
 
       before do
-        command.class.option "--port", "PORT", "port to listen on", :default => 4321
+        command.class.option "--port", "PORT", "port to listen on", default: 4321
       end
 
       it "declares default method" do
-        expect(command.default_port).to eql 4321
+        expect(command.default_port).to eq 4321
       end
 
       describe "#help" do
@@ -112,31 +148,49 @@ describe Clamp::Command do
 
     end
 
+    context "without :default value" do
+
+      before do
+        command.class.option "--port", "PORT", "port to listen on"
+      end
+
+      it "does not declare default method" do
+        expect(command).not_to respond_to(:default_port)
+      end
+
+    end
+
     context "with :multivalued" do
 
       before do
-        command.class.option "--flavour", "FLAVOUR", "flavour(s)", :multivalued => true, :attribute_name => :flavours
+        command.class.option "--flavour", "FLAVOUR", "flavour(s)", multivalued: true, attribute_name: :flavours
       end
 
       it "defaults to empty array" do
-        expect(command.flavours).to eql []
+        expect(command.flavours).to be_empty
       end
 
       it "supports multiple values" do
-        command.parse(%w(--flavour chocolate --flavour vanilla))
-        expect(command.flavours).to eql %w(chocolate vanilla)
+        command.parse(%w[--flavour chocolate --flavour vanilla])
+        expect(command.flavours).to eq %w[chocolate vanilla]
       end
 
       it "generates a single-value appender method" do
         command.append_to_flavours("mud")
         command.append_to_flavours("pie")
-        expect(command.flavours).to eql %w(mud pie)
+        expect(command.flavours).to eq %w[mud pie]
       end
 
       it "generates a multi-value setter method" do
         command.append_to_flavours("replaceme")
-        command.flavours = %w(mud pie)
-        expect(command.flavours).to eql %w(mud pie)
+        command.flavours = %w[mud pie]
+        expect(command.flavours).to eq %w[mud pie]
+      end
+
+      it "does not require a value" do
+        expect do
+          command.parse([])
+        end.not_to raise_error
       end
 
     end
@@ -148,8 +202,8 @@ describe Clamp::Command do
 
       before do
         command.class.option "--port", "PORT", "port to listen on",
-                             :default => 4321,
-                             :environment_variable => "PORT",
+                             default: 4321,
+                             environment_variable: "PORT",
                              &:to_i
         set_env("PORT", environment_value)
         command.parse(args)
@@ -158,7 +212,7 @@ describe Clamp::Command do
       context "when no environment variable is present" do
 
         it "uses the default" do
-          expect(command.port).to eql 4321
+          expect(command.port).to eq 4321
         end
 
       end
@@ -168,15 +222,15 @@ describe Clamp::Command do
         let(:environment_value) { "12345" }
 
         it "uses the environment variable" do
-          expect(command.port).to eql 12_345
+          expect(command.port).to eq 12_345
         end
 
-        context "and a value is specified on the command-line" do
+        context "when a value is specified on the command-line" do
 
-          let(:args) { %w(--port 1500) }
+          let(:args) { %w[--port 1500] }
 
           it "uses command-line value" do
-            expect(command.port).to eql 1500
+            expect(command.port).to eq 1500
           end
 
         end
@@ -198,7 +252,7 @@ describe Clamp::Command do
       let(:environment_value) { nil }
 
       before do
-        command.class.option "--[no-]enable", :flag, "enable?", :default => false, :environment_variable => "ENABLE"
+        command.class.option "--[no-]enable", :flag, "enable?", default: false, environment_variable: "ENABLE"
         set_env("ENABLE", environment_value)
         command.parse([])
       end
@@ -206,33 +260,33 @@ describe Clamp::Command do
       context "when no environment variable is present" do
 
         it "uses the default" do
-          expect(command.enable?).to eql false
+          expect(command.enable?).to be false
         end
 
       end
 
-      %w(1 yes enable on true).each do |truthy_value|
+      %w[1 yes enable on true].each do |truthy_value|
 
         context "when environment variable is #{truthy_value.inspect}" do
 
           let(:environment_value) { truthy_value }
 
           it "sets the flag" do
-            expect(command.enable?).to eql true
+            expect(command.enable?).to be true
           end
 
         end
 
       end
 
-      %w(0 no disable off false).each do |falsey_value|
+      %w[0 no disable off false].each do |falsey_value|
 
         context "when environment variable is #{falsey_value.inspect}" do
 
           let(:environment_value) { falsey_value }
 
           it "clears the flag" do
-            expect(command.enable?).to eql false
+            expect(command.enable?).to be false
           end
 
         end
@@ -244,7 +298,43 @@ describe Clamp::Command do
     context "with :required" do
 
       before do
-        command.class.option "--port", "PORT", "port to listen on", :required => true
+        command.class.option "--port", "PORT", "port to listen on", required: true
+      end
+
+      describe "#help" do
+
+        it "marks it as required" do
+          expect(command.help).to include("port to listen on (required)")
+        end
+
+      end
+
+      context "when no value is provided" do
+
+        it "raises a UsageError" do
+          expect do
+            command.parse([])
+          end.to raise_error(Clamp::UsageError)
+        end
+
+      end
+
+      context "when a value is provided" do
+
+        it "does not raise an error" do
+          expect do
+            command.parse(["--port", "12345"])
+          end.not_to raise_error
+        end
+
+      end
+
+    end
+
+    context "with :required and :multivalued" do
+
+      before do
+        command.class.option "--port", "PORT", "port to listen on", required: true, multivalued: true
       end
 
       context "when no value is provided" do
@@ -277,12 +367,23 @@ describe Clamp::Command do
         end
       end
 
-      it "uses the block to validate and convert the option argument" do
-        expect do
-          command.port = "blah"
-        end.to raise_error(ArgumentError)
-        command.port = "1234"
-        expect(command.port).to eql 1234
+      context "when value is incorrect" do
+
+        it "raises an error" do
+          expect do
+            command.port = "blah"
+          end.to raise_error(ArgumentError)
+        end
+
+      end
+
+      context "when value is convertible" do
+
+        it "uses the block to validate and convert the option argument" do
+          command.port = "1234"
+          expect(command.port).to eq 1234
+        end
+
       end
 
     end
@@ -295,12 +396,12 @@ describe Clamp::Command do
       command.class.option ["-f", "--flavour"], "FLAVOUR", "Flavour of the month"
       command.class.option ["-c", "--color"], "COLOR", "Preferred hue"
       command.class.option ["--scoops"], "N", "Number of scoops",
-                           :default => 1,
-                           :environment_variable => "DEFAULT_SCOOPS" do |arg|
+                           default: 1,
+                           environment_variable: "DEFAULT_SCOOPS" do |arg|
         Integer(arg)
       end
       command.class.option ["-n", "--[no-]nuts"], :flag, "Nuts (or not)\nMay include nuts"
-      command.class.parameter "[ARG] ...", "extra arguments", :attribute_name => :arguments
+      command.class.parameter "[ARG] ...", "extra arguments", attribute_name: :arguments
     end
 
     describe "#parse" do
@@ -309,7 +410,7 @@ describe Clamp::Command do
 
         it "raises a UsageError" do
           expect do
-            command.parse(%w(--foo bar))
+            command.parse(%w[--foo bar])
           end.to raise_error(Clamp::UsageError)
         end
 
@@ -318,13 +419,25 @@ describe Clamp::Command do
       context "with options" do
 
         before do
-          command.parse(%w(--flavour strawberry --nuts --color blue))
+          command.parse(%w[--flavour strawberry --nuts --color blue])
         end
 
-        it "maps the option values onto the command object" do
-          expect(command.flavour).to eql "strawberry"
-          expect(command.color).to eql "blue"
-          expect(command.nuts?).to eql true
+        describe "flavour" do
+          it "maps the option value onto the command object" do
+            expect(command.flavour).to eq "strawberry"
+          end
+        end
+
+        describe "color" do
+          it "maps the option value onto the command object" do
+            expect(command.color).to eq "blue"
+          end
+        end
+
+        describe "nuts?" do
+          it "maps the option value onto the command object" do
+            expect(command.nuts?).to be true
+          end
         end
 
       end
@@ -332,12 +445,19 @@ describe Clamp::Command do
       context "with short options" do
 
         before do
-          command.parse(%w(-f strawberry -c blue))
+          command.parse(%w[-f strawberry -c blue])
         end
 
-        it "recognises short options as aliases" do
-          expect(command.flavour).to eql "strawberry"
-          expect(command.color).to eql "blue"
+        describe "flavour" do
+          it "recognizes short options as aliases" do
+            expect(command.flavour).to eq "strawberry"
+          end
+        end
+
+        describe "color" do
+          it "recognizes short options as aliases" do
+            expect(command.color).to eq "blue"
+          end
         end
 
       end
@@ -345,11 +465,11 @@ describe Clamp::Command do
       context "with a value appended to a short option" do
 
         before do
-          command.parse(%w(-fstrawberry))
+          command.parse(%w[-fstrawberry])
         end
 
         it "works as though the value were separated" do
-          expect(command.flavour).to eql "strawberry"
+          expect(command.flavour).to eq "strawberry"
         end
 
       end
@@ -357,12 +477,19 @@ describe Clamp::Command do
       context "with combined short options" do
 
         before do
-          command.parse(%w(-nf strawberry))
+          command.parse(%w[-nf strawberry])
         end
 
-        it "works as though the options were separate" do
-          expect(command.flavour).to eql "strawberry"
-          expect(command.nuts?).to eql true
+        describe "flavour" do
+          it "works as though the options were separate" do
+            expect(command.flavour).to eq "strawberry"
+          end
+        end
+
+        describe "nuts?" do
+          it "works as though the options were separate" do
+            expect(command.nuts?).to be true
+          end
         end
 
       end
@@ -370,12 +497,39 @@ describe Clamp::Command do
       context "with option arguments attached using equals sign" do
 
         before do
-          command.parse(%w(--flavour=strawberry --color=blue))
+          command.parse(%w[--flavour=strawberry --color=blue])
         end
 
-        it "works as though the option arguments were separate" do
-          expect(command.flavour).to eql "strawberry"
-          expect(command.color).to eql "blue"
+        describe "flavour" do
+          it "maps the option value onto the command object" do
+            expect(command.flavour).to eq "strawberry"
+          end
+        end
+
+        describe "color" do
+          it "maps the option value onto the command object" do
+            expect(command.color).to eq "blue"
+          end
+        end
+
+      end
+
+      context "with option arguments that look like options" do
+
+        before do
+          command.parse(%w[--flavour=-dashing- --scoops -1])
+        end
+
+        describe "flavour" do
+          it "sets the options" do
+            expect(command.flavour).to eq("-dashing-")
+          end
+        end
+
+        describe "scoops" do
+          it "sets the options" do
+            expect(command.scoops).to eq(-1)
+          end
         end
 
       end
@@ -383,8 +537,8 @@ describe Clamp::Command do
       context "with option-like things beyond the arguments" do
 
         it "treats them as positional arguments" do
-          command.parse(%w(a b c --flavour strawberry))
-          expect(command.arguments).to eql %w(a b c --flavour strawberry)
+          command.parse(%w[a b c --flavour strawberry])
+          expect(command.arguments).to eq %w[a b c --flavour strawberry]
         end
 
       end
@@ -395,10 +549,22 @@ describe Clamp::Command do
           command.parse(["foo\n--flavour=strawberry", "bar\n-cblue"])
         end
 
-        it "treats them as positional arguments" do
-          expect(command.arguments).to eql ["foo\n--flavour=strawberry", "bar\n-cblue"]
-          expect(command.flavour).to be_nil
-          expect(command.color).to be_nil
+        describe "arguments" do
+          it "treats them as positional arguments" do
+            expect(command.arguments).to eq ["foo\n--flavour=strawberry", "bar\n-cblue"]
+          end
+        end
+
+        describe "flavour" do
+          it "treats them as positional arguments" do
+            expect(command.flavour).to be_nil
+          end
+        end
+
+        describe "color" do
+          it "treats them as positional arguments" do
+            expect(command.color).to be_nil
+          end
         end
 
       end
@@ -406,8 +572,8 @@ describe Clamp::Command do
       context "with an option terminator" do
 
         it "considers everything after the terminator to be an argument" do
-          command.parse(%w(--color blue -- --flavour strawberry))
-          expect(command.arguments).to eql %w(--flavour strawberry)
+          command.parse(%w[--color blue -- --flavour strawberry])
+          expect(command.arguments).to eq %w[--flavour strawberry]
         end
 
       end
@@ -415,7 +581,7 @@ describe Clamp::Command do
       context "with --flag" do
 
         before do
-          command.parse(%w(--nuts))
+          command.parse(%w[--nuts])
         end
 
         it "sets the flag" do
@@ -428,7 +594,7 @@ describe Clamp::Command do
 
         before do
           command.nuts = true
-          command.parse(%w(--no-nuts))
+          command.parse(%w[--no-nuts])
         end
 
         it "clears the flag" do
@@ -441,7 +607,7 @@ describe Clamp::Command do
 
         it "requests help" do
           expect do
-            command.parse(%w(--help))
+            command.parse(%w[--help])
           end.to raise_error(Clamp::HelpWanted)
         end
 
@@ -451,17 +617,27 @@ describe Clamp::Command do
 
         it "requests help" do
           expect do
-            command.parse(%w(-h))
+            command.parse(%w[-h])
           end.to raise_error(Clamp::HelpWanted)
         end
 
       end
 
+      context "when no option value is provided" do
+
+        it "signals a UsageError" do
+          expect do
+            command.parse(%w[--flavour])
+          end.to raise_error(Clamp::UsageError, /^option '--flavour': no value provided/)
+        end
+
+      end
+
       context "when a bad option value is specified on the command-line" do
 
         it "signals a UsageError" do
           expect do
-            command.parse(%w(--scoops reginald))
+            command.parse(%w[--scoops reginald])
           end.to raise_error(Clamp::UsageError, /^option '--scoops': invalid value for Integer/)
         end
 
@@ -487,8 +663,10 @@ describe Clamp::Command do
       end
 
       it "includes option details" do
-        expect(command.help).to match(/--flavour FLAVOUR +Flavour of the month/)
-        expect(command.help).to match(/--color COLOR +Preferred hue/)
+        flavour_help = "-f, --flavour FLAVOUR +Flavour of the month"
+        color_help = "-c, --color COLOR +Preferred hue"
+
+        expect(command.help).to match(/#{flavour_help}\n +#{color_help}/)
       end
 
       it "handles new lines in option descriptions" do
@@ -505,16 +683,27 @@ describe Clamp::Command do
       command.class.option ["--help"], :flag, "help wanted"
     end
 
-    it "does not generate implicit help option" do
-      expect do
-        command.parse(%w(--help))
-      end.to_not raise_error
-      expect(command.help?).to be true
+    describe "parsing" do
+      it "does not generate implicit help option" do
+        expect do
+          command.parse(%w[--help])
+        end.not_to raise_error
+      end
+    end
+
+    describe "help?" do
+      before do
+        command.parse(%w[--help])
+      end
+
+      it "generates custom help option" do
+        expect(command.help?).to be true
+      end
     end
 
     it "does not recognise -h" do
       expect do
-        command.parse(%w(-h))
+        command.parse(%w[-h])
       end.to raise_error(Clamp::UsageError)
     end
 
@@ -529,12 +718,12 @@ describe Clamp::Command do
     end
 
     it "does not map -h to help" do
-      expect(command.help).to_not match(/-h[, ].*help/)
+      expect(command.help).not_to match(/-h[, ].*help/)
     end
 
-    it "still recognises --help" do
+    it "still recognizes --help" do
       expect do
-        command.parse(%w(--help))
+        command.parse(%w[--help])
       end.to raise_error(Clamp::HelpWanted)
     end
 
@@ -542,22 +731,39 @@ describe Clamp::Command do
 
   describe ".parameter" do
 
-    it "declares option argument accessors" do
-      command.class.parameter "FLAVOUR", "flavour of the month"
-      expect(command.flavour).to eql nil
-      command.flavour = "chocolate"
-      expect(command.flavour).to eql "chocolate"
+    context "when regular" do
+
+      before do
+        command.class.parameter "FLAVOUR", "flavour of the month"
+      end
+
+      context "when not set" do
+        it "equals nil" do
+          expect(command.flavour).to be_nil
+        end
+      end
+
+      context "when set" do
+        before do
+          command.flavour = "chocolate"
+        end
+
+        it "equals the value" do
+          expect(command.flavour).to eq "chocolate"
+        end
+      end
+
     end
 
     context "with explicit :attribute_name" do
 
       before do
-        command.class.parameter "FOO", "a foo", :attribute_name => :bar
+        command.class.parameter "FOO", "a foo", attribute_name: :bar
       end
 
       it "uses the specified attribute_name name to name accessors" do
         command.bar = "chocolate"
-        expect(command.bar).to eql "chocolate"
+        expect(command.bar).to eq "chocolate"
       end
 
     end
@@ -565,11 +771,11 @@ describe Clamp::Command do
     context "with :default value" do
 
       before do
-        command.class.parameter "[ORIENTATION]", "direction", :default => "west"
+        command.class.parameter "[ORIENTATION]", "direction", default: "west"
       end
 
       it "sets the specified default value" do
-        expect(command.orientation).to eql "west"
+        expect(command.orientation).to eq "west"
       end
 
       describe "#help" do
@@ -590,12 +796,23 @@ describe Clamp::Command do
         end
       end
 
-      it "uses the block to validate and convert the argument" do
-        expect do
-          command.port = "blah"
-        end.to raise_error(ArgumentError)
-        command.port = "1234"
-        expect(command.port).to eql 1234
+      context "when value is incorrect" do
+
+        it "raises an error" do
+          expect do
+            command.port = "blah"
+          end.to raise_error(ArgumentError)
+        end
+
+      end
+
+      context "when value is convertible" do
+
+        it "uses the block to validate and convert the argument" do
+          command.port = "1234"
+          expect(command.port).to eq 1234
+        end
+
       end
 
     end
@@ -607,28 +824,46 @@ describe Clamp::Command do
       end
 
       it "accepts multiple arguments" do
-        command.parse(%w(X Y Z))
-        expect(command.file_list).to eql %w(X Y Z)
+        command.parse(%w[X Y Z])
+        expect(command.file_list).to eq %w[X Y Z]
       end
 
     end
 
-    context "optional, with ellipsis" do
+    context "when optional, with ellipsis" do
 
       before do
         command.class.parameter "[FILE] ...", "files"
+
+        command.parse([])
       end
 
-      it "defaults to an empty list" do
-        command.parse([])
-        expect(command.default_file_list).to eql []
-        expect(command.file_list).to eql []
+      describe "default_file_list" do
+
+        it "defaults to an empty list" do
+          expect(command.default_file_list).to be_empty
+        end
+
       end
 
-      it "is mutable" do
-        command.parse([])
-        command.file_list << "treasure"
-        expect(command.file_list).to eql ["treasure"]
+      describe "file_list" do
+
+        it "defaults to an empty list" do
+          expect(command.file_list).to be_empty
+        end
+
+      end
+
+      describe "mutation" do
+
+        before do
+          command.file_list << "treasure"
+        end
+
+        it "is mutable" do
+          expect(command.file_list).to eq ["treasure"]
+        end
+
       end
 
     end
@@ -636,22 +871,20 @@ describe Clamp::Command do
     context "with :environment_variable" do
 
       before do
-        command.class.parameter "[FILE]", "a file", :environment_variable => "FILE",
-                                                    :default => "/dev/null"
+        command.class.parameter "[FILE]", "a file", environment_variable: "FILE",
+                                                    default: "/dev/null"
+
+        set_env("FILE", environment_value)
+        command.parse(args)
       end
 
       let(:args) { [] }
       let(:environment_value) { nil }
 
-      before do
-        set_env("FILE", environment_value)
-        command.parse(args)
-      end
-
       context "when neither argument nor environment variable are present" do
 
         it "uses the default" do
-          expect(command.file).to eql "/dev/null"
+          expect(command.file).to eq File::NULL
         end
 
       end
@@ -663,7 +896,7 @@ describe Clamp::Command do
         describe "and no argument is provided" do
 
           it "uses the environment variable" do
-            expect(command.file).to eql "/etc/motd"
+            expect(command.file).to eq "/etc/motd"
           end
 
         end
@@ -673,7 +906,7 @@ describe Clamp::Command do
           let(:args) { ["/dev/null"] }
 
           it "uses the argument" do
-            expect(command.file).to eql "/dev/null"
+            expect(command.file).to eq File::NULL
           end
 
         end
@@ -715,7 +948,7 @@ describe Clamp::Command do
     before do
       command.class.parameter "X", "x\nxx"
       command.class.parameter "Y", "y"
-      command.class.parameter "[Z]", "z", :default => "ZZZ"
+      command.class.parameter "[Z]", "z", default: "ZZZ"
     end
 
     describe "#parse" do
@@ -726,10 +959,22 @@ describe Clamp::Command do
           command.parse(["crash", "bang", "wallop"])
         end
 
-        it "maps arguments onto the command object" do
-          expect(command.x).to eql "crash"
-          expect(command.y).to eql "bang"
-          expect(command.z).to eql "wallop"
+        describe "x" do
+          it "maps arguments onto the command object" do
+            expect(command.x).to eq "crash"
+          end
+        end
+
+        describe "y" do
+          it "maps arguments onto the command object" do
+            expect(command.y).to eq "bang"
+          end
+        end
+
+        describe "z" do
+          it "maps arguments onto the command object" do
+            expect(command.z).to eq "wallop"
+          end
         end
 
       end
@@ -746,22 +991,52 @@ describe Clamp::Command do
 
       context "with optional argument omitted" do
 
-        it "defaults the optional argument" do
+        before do
           command.parse(["crash", "bang"])
-          expect(command.x).to eql "crash"
-          expect(command.y).to eql "bang"
-          expect(command.z).to eql "ZZZ"
+        end
+
+        describe "x" do
+          it "defaults the optional argument" do
+            expect(command.x).to eq "crash"
+          end
+        end
+
+        describe "y" do
+          it "defaults the optional argument" do
+            expect(command.y).to eq "bang"
+          end
+        end
+
+        describe "z" do
+          it "defaults the optional argument" do
+            expect(command.z).to eq "ZZZ"
+          end
         end
 
       end
 
       context "with multi-line arguments" do
 
-        it "parses them correctly" do
+        before do
           command.parse(["foo\nhi", "bar", "baz"])
-          expect(command.x).to eql "foo\nhi"
-          expect(command.y).to eql "bar"
-          expect(command.z).to eql "baz"
+        end
+
+        describe "x" do
+          it "parses them correctly" do
+            expect(command.x).to eq "foo\nhi"
+          end
+        end
+
+        describe "y" do
+          it "parses them correctly" do
+            expect(command.y).to eq "bar"
+          end
+        end
+
+        describe "z" do
+          it "parses them correctly" do
+            expect(command.z).to eq "baz"
+          end
         end
 
       end
@@ -785,9 +1060,7 @@ describe Clamp::Command do
       end
 
       it "includes parameter details" do
-        expect(command.help).to match(/X +x/)
-        expect(command.help).to match(/Y +y/)
-        expect(command.help).to match(/\[Z\] +z \(default: "ZZZ"\)/)
+        expect(command.help).to match(/X +x\n[\s\S]* +Y +y\n[\s\S]* +\[Z\] +z \(default: "ZZZ"\)/)
       end
 
       it "handles new lines in option descriptions" do
@@ -798,6 +1071,26 @@ describe Clamp::Command do
 
   end
 
+  describe ".execute" do
+
+    before do
+
+      command_class.class_eval do
+        execute do
+          puts "using execute DSL"
+        end
+      end
+
+      command.run([])
+
+    end
+
+    it "provides an alternative way to declare execute method" do
+      expect(stdout).to eq("using execute DSL\n")
+    end
+
+  end
+
   context "with explicit usage" do
 
     given_command("blah") do
@@ -828,8 +1121,7 @@ describe Clamp::Command do
     describe "#help" do
 
       it "includes both potential usages" do
-        expect(command.help).to include("put THIS HERE\n")
-        expect(command.help).to include("put THAT THERE\n")
+        expect(command.help).to match(/put THIS HERE\n +put THAT THERE/)
       end
 
     end
@@ -840,20 +1132,19 @@ describe Clamp::Command do
 
     given_command("punt") do
 
-      banner <<-EOF
+      banner <<-TEXT
         Punt is an example command.  It doesn't do much, really.
 
         The prefix at the beginning of this description should be normalised
         to two spaces.
-      EOF
+      TEXT
 
     end
 
     describe "#help" do
 
       it "includes the banner" do
-        expect(command.help).to match(/^  Punt is an example command/)
-        expect(command.help).to match(/^  The prefix/)
+        expect(command.help).to match(/^  Punt is an example command.+\n^  \n^  The prefix/)
       end
 
     end
@@ -862,28 +1153,42 @@ describe Clamp::Command do
 
   describe ".run" do
 
-    it "creates a new Command instance and runs it" do
-      command.class.class_eval do
-        parameter "WORD ...", "words"
-        def execute
-          print word_list.inspect
+    context "when regular" do
+
+      before do
+
+        command.class.class_eval do
+          parameter "WORD ...", "words"
+          def execute
+            print word_list.inspect
+          end
         end
+
       end
-      @xyz = %w(x y z)
-      command.class.run("cmd", @xyz)
-      expect(stdout).to eql @xyz.inspect
+
+      it "creates a new Command instance and runs it" do
+        xyz = %w[x y z]
+        command.class.run("cmd", xyz)
+        expect(stdout).to eq xyz.inspect
+      end
+
     end
 
-    context "invoked with a context hash" do
+    context "when invoked with a context hash" do
+
+      before do
 
-      it "makes the context available within the command" do
         command.class.class_eval do
           def execute
             print context[:foo]
           end
         end
-        command.class.run("xyz", [], :foo => "bar")
-        expect(stdout).to eql "bar"
+
+      end
+
+      it "makes the context available within the command" do
+        command.class.run("xyz", [], foo: "bar")
+        expect(stdout).to eq "bar"
       end
 
     end
@@ -894,24 +1199,19 @@ describe Clamp::Command do
 
         command.class.class_eval do
           def execute
-            signal_error "Oh crap!", :status => 456
+            signal_error "Oh crap!", status: 456
           end
         end
 
-        begin
-          command.class.run("cmd", [])
-        rescue SystemExit => e
-          @system_exit = e
-        end
-
       end
 
-      it "outputs the error message" do
-        expect(stderr).to include "ERROR: Oh crap!"
-      end
-
-      it "exits with the specified status" do
-        expect(@system_exit.status).to eql 456
+      it "outputs the error message and exits with the specified status" do # rubocop:disable RSpec/ExampleLength
+        expect { command.class.run("cmd", []) }
+          .to raise_error(
+            an_instance_of(SystemExit).and(having_attributes(status: 456))
+          ).and output(<<~TEXT).to_stderr
+            ERROR: Oh crap!
+          TEXT
       end
 
     end
@@ -934,17 +1234,15 @@ describe Clamp::Command do
 
       end
 
-      it "outputs the error message" do
-        expect(stderr).to include "ERROR: bad dog!"
-      end
+      it "outputs the error message and help" do # rubocop:disable RSpec/ExampleLength
+        expect { command.class.run("cmd", []) }
+          .to raise_error(
+            an_instance_of(SystemExit).and(having_attributes(status: 1))
+          ).and output(<<~TEXT).to_stderr
+            ERROR: bad dog!
 
-      it "outputs help" do
-        expect(stderr).to include "See: 'cmd --help'"
-      end
-
-      it "exits with a non-zero status" do
-        expect(@system_exit).to_not be_nil
-        expect(@system_exit.status).to eql 1
+            See: 'cmd --help'
+          TEXT
       end
 
     end
