@@ -1,10 +1,17 @@
+# frozen_string_literal: true
+
 require "rspec"
 require "clamp"
 require "stringio"
+require "pry-byebug"
 
 RSpec.configure do |config|
 
-  config.mock_with :rr
+  config.around do |example|
+    example.run
+  rescue SystemExit => e
+    raise "Unexpected exit with status #{e.status}"
+  end
 
 end
 
